@@ -23,6 +23,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
+#define FGH_PI       3.14159265358979323846
 
 typedef struct
 {
@@ -33,12 +34,15 @@ typedef struct
 AffineTrans ;
 
 /* Number of levels to draw the fractal */
-static int num_levels = 0 ;
+static int num_levels = 4 ;
 
 /* The definition of the fractal */
 static int num_trans ;
 static AffineTrans *affine ;
 
+/* Flag telling us to keep executing the main loop */
+static int continue_in_main_loop = 1;
+
 /* the window title */
 char window_title [ 80 ] ;
 
@@ -125,7 +129,7 @@ Key(unsigned char key, int x, int y)
   
   switch (key) {
   case 27:  /* Escape key */
-    glutLeaveMainLoop () ;
+    continue_in_main_loop = 0 ;
     break;
 
   case '+' :
@@ -208,6 +212,16 @@ Special(int key, int x, int y)
 }
 
 
+static void
+checkedFGets ( char *s, int size, FILE *stream )
+{
+  if ( fgets ( s, size, stream ) == NULL ) {
+    fprintf ( stderr, "fgets failed\n");
+    exit ( EXIT_FAILURE );
+  }
+}
+
+
 void readConfigFile ( char *fnme )
 {
   FILE *fptr = fopen ( fnme, "rt" ) ;
@@ -217,13 +231,13 @@ void readConfigFile ( char *fnme )
   if ( fptr )
   {
     /* Read a header line */
-    fgets ( inputline, 256, fptr ) ;
+    checkedFGets ( inputline, sizeof ( inputline ), fptr ) ;
 
     /* Read a comment line */
-    fgets ( inputline, 256, fptr ) ;
+    checkedFGets ( inputline, sizeof ( inputline ), fptr ) ;
 
     /* Read the window title */
-    fgets ( inputline, 256, fptr ) ;
+    checkedFGets ( inputline, sizeof ( inputline ), fptr ) ;
     /* We assume here that this line will not exceed 79 characters plus a 
        newline (window_title is 80 characters long). That'll cause a buffer 
        overflow. For a simple program like  this, though, we're letting it 
@@ -232,21 +246,21 @@ void readConfigFile ( char *fnme )
     sscanf ( inputline, "%[a-zA-Z0-9!@#$%^&*()+=/\\_-\" ]", window_title ) ; 
 
     /* Read a comment line */
-    fgets ( inputline, 256, fptr ) ;
+    checkedFGets ( inputline, sizeof ( inputline ), fptr ) ;
 
     /* Read the number of affine transformations */
-    fgets ( inputline, 256, fptr ) ;
+    checkedFGets ( inputline, sizeof ( inputline ), fptr ) ;
     sscanf ( inputline, "%d", &num_trans ) ;
 
     affine = (AffineTrans *)malloc ( num_trans * sizeof(AffineTrans) ) ;
 
     /* Read a comment line */
-    fgets ( inputline, 256, fptr ) ;
+    checkedFGets ( inputline, sizeof ( inputline ), fptr ) ;
 
     for ( i = 0; i < num_trans; i++ )
     {
       /* Read an affine transformation definition */
-      fgets ( inputline, 256, fptr ) ;
+      checkedFGets ( inputline, sizeof ( inputline ), fptr ) ;
       sscanf ( inputline, "%lf %lf %lf %lf %lf %lf", &affine[i].a00, &affine[i].a01,
                        &affine[i].a10, &affine[i].a11, &affine[i].b0, &affine[i].b1 ) ;
     }
@@ -254,13 +268,20 @@ void readConfigFile ( char *fnme )
   else  /* No data file, set a default */
   {
     printf ( "ERROR opening file <%s>\n", fnme ) ;
-    strcpy ( window_title, "Cantor Dust" ) ;
-    num_trans = 2 ;
+    strcpy ( window_title, "Koch Snowflake" ) ;
+    num_trans = 4 ;
     affine = (AffineTrans *)malloc ( num_trans * sizeof(AffineTrans) ) ;
-    affine[0].a00 = 0.25 ;  affine[0].a01 = 0.00 ;  affine[0].a10 = 0.00 ;  affine[0].a11 = 0.25 ;
+    affine[0].a00 = 1/3. ;  affine[0].a01 = 0.00 ;  affine[0].a10 = 0.00 ;  affine[0].a11 = 1/3. ;
     affine[0].b0 = 0.0 ;    affine[0].b1 = 0.0 ;
-    affine[1].a00 = 0.25 ;  affine[1].a01 = 0.00 ;  affine[1].a10 = 0.00 ;  affine[1].a11 = 0.25 ;
-    affine[1].b0 = 0.5 ;    affine[1].b1 = 0.0 ;
+    
+    affine[1].a00 = 1/6. ;  affine[1].a01 = -1/3.*sin(FGH_PI/3.) ;   affine[1].a10 = 1/3.*sin(FGH_PI/3.) ;   affine[1].a11 = 1/6. ;
+    affine[1].b0 = 1/3. ;   affine[1].b1 = 0.0 ;
+
+    affine[2].a00 = 1/6. ;  affine[2].a01 = -1/3.*sin(-FGH_PI/3.) ;  affine[2].a10 = 1/3.*sin(-FGH_PI/3.) ;  affine[2].a11 = 1/6. ;
+    affine[2].b0 = 0.5 ;    affine[2].b1 = sqrt(3)/6. ;
+
+    affine[3].a00 = 1/3. ;  affine[3].a01 = 0.00 ;  affine[3].a10 = 0.00 ;  affine[3].a11 = 1/3. ;
+    affine[3].b0 = 2/3. ;   affine[3].b1 = 0.0 ;
   }
 
   for ( i = 0; i < num_trans; i++ )
@@ -311,7 +332,11 @@ main(int argc, char *argv[])
   glutSpecialFunc(Special);
   glutDisplayFunc(Display);
 
-  glutMainLoop();
+#ifdef WIN32
+#endif
+
+  while ( continue_in_main_loop )
+    glutMainLoopEvent();
 
   printf ( "Back from the 'freeglut' main loop\n" ) ;
 
