@@ -22,7 +22,8 @@
 #include <GL/freeglut.h>
 
 int g_LeaveGameMode = 0;
-int g_InGameMode = 1;
+int g_InGameMode = 0;
+int g_mainwin1, g_mainwin2, g_sw1, g_sw2, g_gamemodewin;
 
 /*
  * Call this function to have some text drawn at given coordinates
@@ -93,54 +94,86 @@ void PrintText( int nX, int nY, char* pszText )
 /*
  * This is the display routine for our sample FreeGLUT windows
  */
-static float g_fTime = 0.0f;
-
 void SampleDisplay( void )
 {
-    /*
-     * Clear the screen
-     */
-    glClearColor( 0, 0.5, 1, 1 );
-    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
+    int win = glutGetWindow();
 
-    /*
-     * Have the cube rotated
-     */
-    glMatrixMode( GL_MODELVIEW );
-    glPushMatrix();
+    if (g_InGameMode && win!=g_gamemodewin)
+        /* Don't draw other windows when in gamemode, those aren't visible
+         * anyway. Drawing them continuously nonetheless can cause flicker trouble
+         * on my machine. This only seems to occur only when there are child windows
+         * among the non-visible windows 
+         */
+        return;
 
-    glRotatef( g_fTime, 0, 0, 1 );
-    glRotatef( g_fTime, 0, 1, 0 );
-    glRotatef( g_fTime, 1, 0, 0 );
-
-    /*
-     * And then drawn...
-     */
-    glColor3f( 1, 1, 0 );
-    /* glutWireCube( 20.0 ); */
-    glutWireTeapot( 20.0 );
-    /* glutWireSpher( 15.0, 15, 15 ); */
-    /* glColor3f( 0, 1, 0 ); */
-    /* glutWireCube( 30.0 ); */
-    /* glutSolidCone( 10, 20, 10, 2 ); */
-
-    /*
-     * Don't forget about the model-view matrix
-     */
-    glPopMatrix( );
-
-    /*
-     * Draw a silly text
-     */
-    if( g_InGameMode == 0 )
-        PrintText( 20, 20, "Hello there cruel world!" );
+    if (win==g_sw1)
+    {
+        /*
+         * Clear the screen
+         */
+        glClearColor(0.7f,0.7f,0.7f,1);
+        glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
+        glutPostWindowRedisplay(g_mainwin2);
+    }
+    else if (win==g_sw2)
+    {
+        /*
+         * Clear the screen
+         */
+        glClearColor(0.3f,0.3f,0.3f,1);
+        glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
+        glutPostWindowRedisplay(g_mainwin2);
+    }
     else
-        PrintText( 20, 20, "Press ESC to leave the game mode!" );
+    {
+        const GLfloat time = glutGet(GLUT_ELAPSED_TIME) / 1000.f * 40;
+
+        /*
+         * Clear the screen
+         */
+        glClearColor( 0, 0.5, 1, 1 );
+        glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
+
+        /*
+         * Have the cube rotated
+         */
+        glMatrixMode( GL_MODELVIEW );
+        glPushMatrix();
+
+        glRotatef( time, 0, 0, 1 );
+        glRotatef( time, 0, 1, 0 );
+        glRotatef( time, 1, 0, 0 );
+
+        /*
+         * And then drawn...
+         */
+        glColor3f( 1, 1, 0 );
+        /* glutWireCube( 20.0 ); */
+        glutWireTeapot( 20.0 );
+        /* glutWireSphere( 15.0, 15, 15 ); */
+        /* glColor3f( 0, 1, 0 ); */
+        /* glutWireCube( 30.0 ); */
+        /* glutSolidCone( 10, 20, 10, 2 ); */
+
+        /*
+         * Don't forget about the model-view matrix
+         */
+        glPopMatrix( );
+
+        /*
+         * Draw a silly text
+         */
+        if( g_InGameMode == 0 )
+            PrintText( 20, 20, "Hello there cruel world!" );
+        else
+            PrintText( 20, 20, "Press ESC to leave the game mode!" );
+    }
 
     /*
      * And swap this context's buffers
      */
     glutSwapBuffers( );
+    glutPostWindowRedisplay(win);
 }
 
 /*
@@ -148,16 +181,26 @@ void SampleDisplay( void )
  */
 void SampleIdle( void )
 {
-    g_fTime += 0.5f;
-
     if( g_LeaveGameMode == 1 )
     {
+        /* One could do all this just as well in SampleGameModeKeyboard... */
+        printf("leaving gamemode...\n");
         glutLeaveGameMode( );
         g_LeaveGameMode = 0;
         g_InGameMode = 0;
+        glutPostWindowRedisplay(g_mainwin1);
+        glutPostWindowRedisplay(g_mainwin2);
+        glutPostWindowRedisplay(g_sw1);
+        glutPostWindowRedisplay(g_sw2);
     }
 }
 
+void SampleEntry(int state)
+{
+    int window = glutGetWindow () ;
+    printf ( "Window %d Entry Callback: %d\n", window, state ) ;
+}
+
 /*
  * The reshape function
  */
@@ -239,12 +282,17 @@ void SampleSpecial( int nSpecial, int nMouseX, int nMouseY )
  */
 void SampleMenu( int menuID )
 {
-    /*
-     * Just print something funny
-     */
     printf( "SampleMenu() callback executed, menuID is %i\n", menuID );
 }
 
+/*
+ * A sample menu status callback
+ */
+void SampleMenuStatus( int status, int x, int y )
+{
+    printf ( "SampleMenu() callback executed, MenuStatus is %i at (%i,%i)\n", status, x, y );
+}
+
 /*
  * The sample's entry point
  */
@@ -258,6 +306,8 @@ int main( int argc, char** argv )
 
     glutInit( &argc, argv );
 
+    glutSetOption(GLUT_ACTION_ON_WINDOW_CLOSE,GLUT_ACTION_GLUTMAINLOOP_RETURNS);
+
     subMenuA = glutCreateMenu( SampleMenu );
     glutAddMenuEntry( "Sub menu A1 (01)", 1 );
     glutAddMenuEntry( "Sub menu A2 (02)", 2 );
@@ -278,22 +328,38 @@ int main( int argc, char** argv )
     glutAddSubMenu( "Enter sub menu A", subMenuA );
     glutAddSubMenu( "Enter sub menu B", subMenuB );
 
-    glutCreateWindow( "Hello world!" );
+    g_mainwin1 = glutCreateWindow( "Hello world!" );
     glutDisplayFunc( SampleDisplay );
     glutReshapeFunc( SampleReshape );
     glutKeyboardFunc( SampleKeyboard );
     glutSpecialFunc( SampleSpecial );
     glutIdleFunc( SampleIdle );
+    glutEntryFunc( SampleEntry );
+    glutMenuStatusFunc( SampleMenuStatus );
     glutAttachMenu( GLUT_LEFT_BUTTON );
 
     glutInitWindowPosition( 200, 200 );
-    glutCreateWindow( "I am not Jan B." );
+    g_mainwin2 = glutCreateWindow( "I am not Jan B." );
     glutDisplayFunc( SampleDisplay );
     glutReshapeFunc( SampleReshape );
     glutKeyboardFunc( SampleKeyboard );
     glutSpecialFunc( SampleSpecial );
     glutIdleFunc( SampleIdle );
+    glutEntryFunc( SampleEntry );
+    glutMenuStatusFunc( SampleMenuStatus );
     glutAttachMenu( GLUT_LEFT_BUTTON );
+    glutSetMenu(subMenuA);
+    glutAttachMenu( GLUT_RIGHT_BUTTON );
+
+    g_sw1=glutCreateSubWindow(g_mainwin2,200,0,100,100);
+    glutDisplayFunc( SampleDisplay );
+    glutSetMenu(subMenuB);
+    glutAttachMenu( GLUT_LEFT_BUTTON );
+
+    g_sw2=glutCreateSubWindow(g_sw1,50,0,50,50);
+    glutDisplayFunc( SampleDisplay );
+    glutSetMenu(menuID);
+    glutAttachMenu( GLUT_RIGHT_BUTTON );
 
     printf( "Testing game mode string parsing, don't panic!\n" );
     glutGameModeString( "320x240:32@100" );
@@ -301,18 +367,27 @@ int main( int argc, char** argv )
     glutGameModeString( "1024x768" );
     glutGameModeString( ":32@120" );
     glutGameModeString( "Toudi glupcze, Danwin bedzie moj!" );
-    glutGameModeString( "640x480:16@72" );
-
+    
+    glutGameModeString( "640x480:37@300" );    /* this one should fail */
     glutEnterGameMode();
+
+    glutGameModeString( "800x600" );    /* this one is likely to succeed */
+    g_gamemodewin = glutEnterGameMode();
+
+    if (glutGameModeGet(GLUT_GAME_MODE_ACTIVE))
+        g_InGameMode = 1;
     glutDisplayFunc( SampleDisplay );
     glutReshapeFunc( SampleReshape );
     glutKeyboardFunc( SampleGameModeKeyboard );
     glutIdleFunc( SampleIdle );
+    glutEntryFunc( SampleEntry );
+    glutMenuStatusFunc( SampleMenuStatus );
+    glutSetMenu(menuID);
     glutAttachMenu( GLUT_LEFT_BUTTON );
 
-    printf( "current window is %ix%i+%i+%i",
-            glutGet( GLUT_WINDOW_X ), glutGet( GLUT_WINDOW_Y ),
-            glutGet( GLUT_WINDOW_WIDTH ), glutGet( GLUT_WINDOW_HEIGHT )
+    printf( "current window is %ix%i at (%i,%i)\n",
+        glutGet( GLUT_WINDOW_WIDTH ), glutGet( GLUT_WINDOW_HEIGHT ),
+        glutGet( GLUT_WINDOW_X ), glutGet( GLUT_WINDOW_Y )
     );
 
     /*
@@ -320,11 +395,12 @@ int main( int argc, char** argv )
      */
     glutMainLoop();
 
+    /*
+     * returned from mainloop after window closed
+     * see glutSetOption(GLUT_ACTION_ON_WINDOW_CLOSE,GLUT_ACTION_GLUTMAINLOOP_RETURNS); above
+     */
     printf( "glutMainLoop() termination works fine!\n" );
 
-    /*
-     * This is never reached in FreeGLUT. Is that good?
-     */
     return EXIT_SUCCESS;
 }
 
