@@ -28,10 +28,6 @@
 #include <GL/freeglut.h>
 #include "freeglut_internal.h"
 
-#if TARGET_HOST_UNIX_X11
-  #include <X11/cursorfont.h>
-#endif
-
 /*
  * TODO BEFORE THE STABLE RELEASE:
  *  glutSetCursor()     -- Win32 mappings are incomplete.
@@ -43,7 +39,9 @@
 
 /* -- PRIVATE FUNCTIONS --------------------------------------------------- */
 
-#if TARGET_HOST_UNIX_X11
+#if TARGET_HOST_POSIX_X11 || TARGET_HOST_MAC_OSX || TARGET_HOST_SOLARIS
+  #include <X11/cursorfont.h>
+
 /*
  * A factory method for an empty cursor
  */
@@ -102,64 +100,92 @@ static cursorCacheEntry cursorCache[] = {
     { XC_bottom_right_corner, None }, /* GLUT_CURSOR_BOTTOM_RIGHT_CORNER */
     { XC_bottom_left_corner,  None }  /* GLUT_CURSOR_BOTTOM_LEFT_CORNER */
 };
-#endif
 
-/* -- INTERNAL FUNCTIONS ---------------------------------------------------- */
-
-/*
- * Set the cursor image to be used for the current window
- */
-void fgSetCursor ( SFG_Window *window, int cursorID )
+static void fghSetCursor ( SFG_Window *window, int cursorID )
 {
-#if TARGET_HOST_UNIX_X11
-    {
-        Cursor cursor;
-        /*
-         * XXX FULL_CROSSHAIR demotes to plain CROSSHAIR. Old GLUT allows
-         * for this, but if there is a system that easily supports a full-
-         * window (or full-screen) crosshair, we might consider it.
-         */
-        int cursorIDToUse =
-            ( cursorID == GLUT_CURSOR_FULL_CROSSHAIR ) ? GLUT_CURSOR_CROSSHAIR : cursorID;
+    Cursor cursor;
+    /*
+     * XXX FULL_CROSSHAIR demotes to plain CROSSHAIR. Old GLUT allows
+     * for this, but if there is a system that easily supports a full-
+     * window (or full-screen) crosshair, we might consider it.
+     */
+    int cursorIDToUse =
+        ( cursorID == GLUT_CURSOR_FULL_CROSSHAIR ) ? GLUT_CURSOR_CROSSHAIR : cursorID;
 
-        if( ( cursorIDToUse >= 0 ) &&
-            ( cursorIDToUse < sizeof( cursorCache ) / sizeof( cursorCache[0] ) ) ) {
-            cursorCacheEntry *entry = &cursorCache[ cursorIDToUse ];
-            if( entry->cachedCursor == None ) {
-                entry->cachedCursor =
-                    XCreateFontCursor( fgDisplay.Display, entry->cursorShape );
-            }
-            cursor = entry->cachedCursor;
-        } else {
-            switch( cursorIDToUse )
-            {
-            case GLUT_CURSOR_NONE:
-                cursor = getEmptyCursor( );
-                break;
-
-            case GLUT_CURSOR_INHERIT:
-                cursor = None;
-                break;
-
-            default:
-                fgError( "Unknown cursor type: %d", cursorIDToUse );
-                return;
-            }
+    if( ( cursorIDToUse >= 0 ) &&
+        ( cursorIDToUse < sizeof( cursorCache ) / sizeof( cursorCache[0] ) ) ) {
+        cursorCacheEntry *entry = &cursorCache[ cursorIDToUse ];
+        if( entry->cachedCursor == None ) {
+            entry->cachedCursor =
+                XCreateFontCursor( fgDisplay.Display, entry->cursorShape );
         }
+        cursor = entry->cachedCursor;
+    } else {
+        switch( cursorIDToUse )
+        {
+        case GLUT_CURSOR_NONE:
+            cursor = getEmptyCursor( );
+            break;
 
-        if ( ( cursorIDToUse != GLUT_CURSOR_NONE ) && ( cursor == None ) ) {
-            fgError( "Failed to create cursor" );
+        case GLUT_CURSOR_INHERIT:
+            cursor = None;
+            break;
+
+        default:
+            fgError( "Unknown cursor type: %d", cursorIDToUse );
+            return;
         }
-        XDefineCursor( fgDisplay.Display,
-                       window->Window.Handle, cursor );
     }
 
-#elif TARGET_HOST_WIN32 || TARGET_HOST_WINCE
+    if ( cursorIDToUse == GLUT_CURSOR_INHERIT ) {
+        XUndefineCursor( fgDisplay.Display, window->Window.Handle );
+    } else if ( cursor != None ) {
+        XDefineCursor( fgDisplay.Display, window->Window.Handle, cursor );
+    } else if ( cursorIDToUse != GLUT_CURSOR_NONE ) {
+        fgError( "Failed to create cursor" );
+    }
+}
 
+
+static void fghWarpPointer ( int x, int y )
+{
+    XWarpPointer(
+        fgDisplay.Display,
+        None,
+        fgStructure.CurrentWindow->Window.Handle,
+        0, 0, 0, 0,
+        x, y
+    );
+    /* Make the warp visible immediately. */
+    XFlush( fgDisplay.Display );
+}
+
+void fghGetCursorPos(SFG_XYUse *mouse_pos)
+{
+    /* Get current pointer location in screen coordinates
+    */
+    Window junk_window;
+    unsigned int junk_mask;
+    int junk_pos;
+    
+    XQueryPointer(fgDisplay.Display, fgDisplay.RootWindow,
+            &junk_window, &junk_window,
+            &mouse_pos->X, &mouse_pos->Y,
+            &junk_pos, &junk_pos, &junk_mask);
+    
+    mouse_pos->Use = GL_TRUE;
+}
+#endif
+
+
+#if TARGET_HOST_MS_WINDOWS
+static void fghSetCursor ( SFG_Window *window, int cursorID )
+{
     /*
-     * This is a temporary solution only...
+     * Joe Krahn is re-writing the following code.
      */
     /* Set the cursor AND change it for this window class. */
+#if !defined(__MINGW64__) && _MSC_VER <= 1200
 #       define MAP_CURSOR(a,b)                                   \
         case a:                                                  \
             SetCursor( LoadCursor( NULL, b ) );                  \
@@ -167,7 +193,6 @@ void fgSetCursor ( SFG_Window *window, int cursorID )
                           GCL_HCURSOR,                           \
                           ( LONG )LoadCursor( NULL, b ) );       \
         break;
-
     /* Nuke the cursor AND change it for this window class. */
 #       define ZAP_CURSOR(a,b)                                   \
         case a:                                                  \
@@ -175,6 +200,22 @@ void fgSetCursor ( SFG_Window *window, int cursorID )
             SetClassLong( window->Window.Handle,                 \
                           GCL_HCURSOR, ( LONG )NULL );           \
         break;
+#else
+#       define MAP_CURSOR(a,b)                                   \
+        case a:                                                  \
+            SetCursor( LoadCursor( NULL, b ) );                  \
+            SetClassLongPtr( window->Window.Handle,              \
+                          GCLP_HCURSOR,                          \
+                          ( LONG )( LONG_PTR )LoadCursor( NULL, b ) );       \
+        break;
+    /* Nuke the cursor AND change it for this window class. */
+#       define ZAP_CURSOR(a,b)                                   \
+        case a:                                                  \
+            SetCursor( NULL );                                   \
+            SetClassLongPtr( window->Window.Handle,              \
+                          GCLP_HCURSOR, ( LONG )( LONG_PTR )NULL );          \
+        break;
+#endif
 
     switch( cursorID )
     {
@@ -206,11 +247,41 @@ void fgSetCursor ( SFG_Window *window, int cursorID )
         fgError( "Unknown cursor type: %d", cursorID );
         break;
     }
+}
+
+
+static void fghWarpPointer ( int x, int y )
+{
+    POINT coords;
+    coords.x = x;
+    coords.y = y;
+
+    /* ClientToScreen() translates {coords} for us. */
+    ClientToScreen( fgStructure.CurrentWindow->Window.Handle, &coords );
+    SetCursorPos( coords.x, coords.y );
+}
+
+void fghGetCursorPos(SFG_XYUse *mouse_pos)
+{
+    /* Get current pointer location in screen coordinates
+     */
+    POINT pos;
+    GetCursorPos(&pos);
+
+    mouse_pos->X = pos.x;
+    mouse_pos->Y = pos.y;
+    mouse_pos->Use = GL_TRUE;
+}
 #endif
 
-    window->State.Cursor = cursorID;
+
+/* -- INTERNAL FUNCTIONS ---------------------------------------------------- */
+void fgSetCursor ( SFG_Window *window, int cursorID )
+{
+    fghSetCursor ( window, cursorID );
 }
 
+
 /* -- INTERFACE FUNCTIONS -------------------------------------------------- */
 
 /*
@@ -221,7 +292,8 @@ void FGAPIENTRY glutSetCursor( int cursorID )
     FREEGLUT_EXIT_IF_NOT_INITIALISED ( "glutSetCursor" );
     FREEGLUT_EXIT_IF_NO_WINDOW ( "glutSetCursor" );
 
-    fgSetCursor ( fgStructure.CurrentWindow, cursorID );
+    fghSetCursor ( fgStructure.CurrentWindow, cursorID );
+    fgStructure.CurrentWindow->State.Cursor = cursorID;
 }
 
 /*
@@ -232,31 +304,7 @@ void FGAPIENTRY glutWarpPointer( int x, int y )
     FREEGLUT_EXIT_IF_NOT_INITIALISED ( "glutWarpPointer" );
     FREEGLUT_EXIT_IF_NO_WINDOW ( "glutWarpPointer" );
 
-#if TARGET_HOST_UNIX_X11
-
-    XWarpPointer(
-        fgDisplay.Display,
-        None,
-        fgStructure.CurrentWindow->Window.Handle,
-        0, 0, 0, 0,
-        x, y
-    );
-    /* Make the warp visible immediately. */
-    XFlush( fgDisplay.Display );
-
-#elif TARGET_HOST_WIN32 || TARGET_HOST_WINCE
-
-    {
-        POINT coords;
-        coords.x = x;
-        coords.y = y;
-
-        /* ClientToScreen() translates {coords} for us. */
-        ClientToScreen( fgStructure.CurrentWindow->Window.Handle, &coords );
-        SetCursorPos( coords.x, coords.y );
-    }
-
-#endif
+    fghWarpPointer ( x, y );
 }
 
 /*** END OF FILE ***/
